// Importing required functions from d3 library
import { csv, select } from 'd3';
// Importing scatterPlot function from a local module
import { scatterPlot } from './scatterPlot';

// Define the URL to retrieve the diabetes dataset (assuming it's available online)
const csvUrl =
  'https://gist.githubusercontent.com/arun-thotakuri/4cfeeb36976ec98561685576680c3465/raw/ef9e6f6ada4185e3b39c1d70e433bc35e591fe28/diabates.csv'; // Update with the actual URL

// Function to parse each CSV row and convert specific attributes to numbers
const parseRow = (d) => {
  d.pregnancies = +d.pregnancies;
  d.bmi = +d.bmi;
  d.skin_thickness = +d.skin_thickness;
  d.glucose = +d.glucose;
  d.age = +d.age;
  d.outcome = +d.outcome;
  return d;
};

// Adjusting the SVG canvas's width and height to fit the dimensions of the window
const width = window.innerWidth;
const height = window.innerHeight - 70;

// Creating an SVG element with a defined width and height attached to the body of the page
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// Main asynchronous function for data loading and plot creation
const main = async () => {
  // Instantiating a scatter plot with specified settings
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow)) // Parsing and loading information from the CSV URL
    .xValue((d) => d.age) // Value accessor for X-axis (age)
    .yValue((d) => d.glucose) // Value accessor for Y-axis (glucose)
    .colorValue((d) => d.outcome) // Value accessor for coloring points based on outcome
    .margin({
      // Determining the plot's margins
      top: 20,
      right: 20,
      bottom: 40,
      left: 50,
    })
    .radius(5); // The scatter plot points' radius

  // Plot attachment to SVG element
  svg.call(plot);

  // The x-axis value is an array of column names that must be cycled through.
  const columns = [
    'pregnancies',
    'bmi',
    'skin_thickness',
    'glucose',
    'age',
    'outcome',
  ];

  // Setting up a counter so that it can cycle over the columns
  let i = 0;

  // Establishing a time interval to redraw the plot and update the x-axis value every two seconds
  setInterval(() => {
    plot.xValue((d) => d[columns[i % columns.length]]); // Altering the value accessor for the x-axis
    svg.call(plot); // Plotting again using the updated x-axis values
    i++; // Increasing the counter correspondingly
  }, 2000);
};

// Carrying out the main task to develop the plot
main();
