import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  select,
} from 'd3';

// Function to determine SVG path for different shapes
const shapePath = (d, radius) => {
  const shapes = {
    Adelie: `M${-radius},${-radius}h${radius * 2}v${radius * 2}h${-radius * 2}Z`, // square
    Chinstrap: `M0,${-radius}L${-radius},${radius}H${radius}Z`, // triangle
    Gentoo: `M0,${-radius}A${radius},${radius} 0 1,1 0,${radius}A${radius},${radius} 0 1,1 0,${-radius}`, // circle
  };
  return shapes[d.species] || shapes['Gentoo']; // default to circle if species is not found
};

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let speciesValue; // Add a species value accessor
  let margin;
  let radius;

  const my = (selection) => {
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      species: speciesValue(d), // get species value
    }));

    selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('d', (d) => shapePath(d, radius)) // set SVG path based on species
      .attr('transform', (d) => `translate(${d.x},${d.y})`);

    // TODO make it idempotent
    // We'll do this live on May 22, 2021!
    // Tune in here: https://youtu.be/U8x862hBk4s

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`,
      )
      .call(axisBottom(x));
  };

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.speciesValue = function (_) {
    // method for species value accessor
    return arguments.length
      ? ((speciesValue = _), my)
      : speciesValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
